// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-A09
// Copyright IBM Corporation 2010. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

// Wrapper class around the DatePicker to have to entry fields: one for date, one for time

/*global dojo,com,dijit,console */

dojo.provide("DateTimePicker");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");

dojo.declare("DateTimePicker", [dijit._Widget, dijit._Templated], {
    templateString: '' +
    '<div dojoAttachPoint="pickerDiv" id="widget_${id}">' +
    '  <span dojoAttachPoint="leftPicker"></span>' +
    '  <span dojoAttachPoint="rightPicker"></span>' +
    '  <span dojoAttachPoint="calImage" style="position: absolute;"></span>' +
    '  <input dojoAttachPoint="hiddenInput" style="display:none" id=${id} name=${name} type="text" isDateField="true" />' +
    '</div>',

    defaults: {
        datePattern: "yyyy-MMM-dd",
        timePattern: "HH:mm:ss",
        imageURL: "images/calendar.gif"
    },
    datePattern: "",
    timePattern: "",
    datePatternRegEx: /y|M|E|e|D|d/g,
    timePatternRegEx: /K|k|H|h|m|s|S|a/g,
    showDateWidget: true,
    showTimeWidget: true,
    constructor: function(args){
        if(args.dateTimePattern) {
            //Assuming date time format will come in as a date pattern only, time pattern only,
            //  or a <date_pattern> <time_pattern> (separated by a space)

            var pattern = args.dateTimePattern;

            var firstSpaceIndex = pattern.indexOf(" ");
            // Since some time formats can have spaces in them, check for a space, the check to see that the dateTimePattern
            // matches both the time and date patterns.
            if(firstSpaceIndex > -1 && pattern.match(this.datePatternRegEx) && pattern.match(this.timePatternRegEx)) {
                this.datePattern = pattern.substring(0, firstSpaceIndex);
                if(firstSpaceIndex+1 <= pattern.length) {
                    this.timePattern = pattern.substring(firstSpaceIndex+1);
                }
            } else {
                if(pattern.match(this.datePatternRegEx)) {
                    this.datePattern = pattern;
                    this.showTimeWidget = false;
                } else {
                    this.timePattern = pattern;
                    this.showDateWidget = false;
                }
            }
        }
        
        var options = {};
        if (args.value) {
            options.value = args.value;
        }

        // Note: do not pass args to child widgets, else we ge dup ids, and the children can't register with dijit
        this.dateBoxWidget = new dijit.form.DateTextBox(options);
        this.dateBoxWidget.constraints.datePattern = this.datePattern;

        this.timeBoxWidget = new dijit.form.TimeTextBox();
        this.timeBoxWidget.constraints.timePattern = this.timePattern;
        this.timeBoxWidget.value = this.dateBoxWidget.value;
    },

    postCreate: function(){
        if(this.showDateWidget) {
            this.dateBoxWidget.placeAt(this.leftPicker);
        }
        if(this.showTimeWidget) {
            this.timeBoxWidget.placeAt(this.rightPicker);
        }
        dojo.place(dojo.create("img",{src: this.defaults.imageURL}),this.calImage);

        if (this.showDateWidget) {
        	if (this.showTimeWidget) {
        		dojo.addClass(this.dateBoxWidget.domNode, 'dateSelectionDoubleInputTextBox');
        		dojo.addClass(this.timeBoxWidget.domNode, 'dateSelectionDoubleInputTextBox');
        	} else {
        		dojo.addClass(this.dateBoxWidget.domNode, 'dateSelectionSingleInputTextBox');
        	}
        } else if (this.showTimeWidget) {
        	dojo.addClass(this.timeBoxWidget.domNode, 'dateSelectionSingleInputTextBox');
        }
        
        this.timeBoxWidget.startup();

        this.connect(this.timeBoxWidget, "onChange", "syncDateToInputField");
        this.connect(this.dateBoxWidget, "onChange", "syncDateToInputField");
    },

    // overwrite the attr function when someone whats to retrive the value
    attr: function attr(name, val){
        var rc;
        if (name == "value") {
            if (val == undefined || val == "" || val == null) {
                // someone wants to read value. Orig. attr returns only date
                this._ensureValuesAreSet();

                this.dateBoxWidget.value.setHours(this.timeBoxWidget.value.getHours());
                this.dateBoxWidget.value.setMinutes(this.timeBoxWidget.value.getMinutes());
                this.dateBoxWidget.value.setSeconds(this.timeBoxWidget.value.getSeconds());
                rc = this.dateBoxWidget.value;
            }
            else {
                var options = {};
                if(this.showDateWidget) { options.datePattern = this.datePattern; }
                if(this.showTimeWidget) { options.timePattern = this.timePattern; }

                // If one is hidden, specify the selector so that the parser will work correctly otherwise it assumes "<date> <time>"
                if(this.showDateWidget != this.showTimeWidget) {
                    options.selector = this.showDateWidget ? "date" : "time";
                }

                var value = dojo.date.locale.parse(val, options);
                if(value == null) {
                    return;
                }
                // set the date and time
                this.timeBoxWidget.attr("value", value); // time part
                rc = this.dateBoxWidget.attr("value", value); // date part
            }
        }
        return rc;
    },

    setValue: function(value){
        this.attr("value", value);
    },
    getValue: function(){
        return this.attr("value");
    },

    // sync the date picker to the hidden input
    syncDateToInputField: function(){
        var dateFormat = {
            datePattern: this.datePattern,
            selector: "date"
        };
        var timeFormat = {
            timePattern: this.timePattern,
            selector: "time"
        };

        this._ensureValuesAreSet;

        // If we are only showing one of the widgets, only write output for the one we are showing
        if(this.showDateWidget != this.showTimeWidget) {
            this.hiddenInput.value = this.showDateWidget ?
                    this.dateBoxWidget.format(this.dateBoxWidget.value, dateFormat) :
                    this.timeBoxWidget.format(this.timeBoxWidget.value, timeFormat);
        } else {
            this.hiddenInput.value=this.dateBoxWidget.format(this.dateBoxWidget.value, dateFormat)
                + " " + this.timeBoxWidget.format(this.timeBoxWidget.value, timeFormat);
        }
    },

    _ensureValuesAreSet : function() {
        // default time settings: current day, 0:00
        if (!this.dateBoxWidget.value) {
            this.dateBoxWidget.value = new Date();
        }
        if (!this.timeBoxWidget.value) {
            this.timeBoxWidget.value = new Date(0, 0, 0, 0, 0, 0, 0);
        }
    },

    isValid : function() {
        return this.dateBoxWidget.isValid() && this.timeBoxWidget.isValid();
    },
    
    setDisabled : function(disabled) {
        if (this.dateBoxWidget) this.dateBoxWidget.setDisabled(disabled);
        if (this.timeBoxWidget) this.timeBoxWidget.setDisabled(disabled);
    },
 
    addEventHandler : function(event, method) {
        this.connect(this.timeBoxWidget, event, method);
        this.connect(this.dateBoxWidget, event, method);
    }
});
